@@ -1,5 +1,4 @@
-
-OUTPUT_DIR_STR = 'output-dir'
+OUTPUT_DIR_STR = 'output_dir'
 SLICE_STR = 'slice'
 NAME_STR = 'name'
 FILE_STR = 'file'
