@@ -47,7 +47,7 @@ def validate_checksum(packet_without_sync):
     is_valid = (ck_a == ck_a_ref and ck_b == ck_b_ref)
     if not is_valid:
         msg_str = helpers.print_bytes(packet_without_sync)
-        logger.critical('Invalid checksum for packet [ {} ] (nbytes {}), expected [ {} ], got [ {} ]'.format(msg_str, len(packet_without_sync), bytes([ck_a_ref, ck_b_ref]), bytes([ck_a, ck_b])))
+        logger.debug('Invalid checksum for packet [ {} ] (nbytes {}), expected [ {} ], got [ {} ]'.format(msg_str, len(packet_without_sync), bytes([ck_a_ref, ck_b_ref]), bytes([ck_a, ck_b])))
 
     return is_valid
 
@@ -59,8 +59,6 @@ def ublox_packets(stream, timeout):
         packet = get_next_package(stream)
         if packet:
             yield packet
-        else:
-            sys.stderr.write("Invalid checksum\n")
 
 
 def get_next_package(stream):
