@@ -5,6 +5,8 @@ import struct
 import tempfile
 import time
 import threading
+import sys
+import glob
 
 import serial
 
@@ -16,46 +18,110 @@ from . import config as ublox_config
 from . import core
 from . import SERIAL_PORT_STR, BAUD_RATE_STR, DEFAULT_BAUD_RATE
 
-def autodetect_serial_port():
+def scan_serial_ports():
+    """ Lists serial port names
+
+        :raises EnvironmentError:
+            On unsupported or unknown platforms
+        :returns:
+            A list of the serial ports available on the system
+    """
+    if sys.platform.startswith('win'):
+        ports = ['COM%s' % (i + 1) for i in range(256)]
+    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
+        # this excludes your current terminal "/dev/tty"
+        ports = glob.glob('/dev/tty[A-Za-z]*')
+    elif sys.platform.startswith('darwin'):
+        ports = glob.glob('/dev/tty.*')
+    else:
+        raise EnvironmentError('Unsupported platform')
+
+    result = []
+    for port in ports:
+        try:
+            s = serial.Serial(port)
+            s.close()
+            result.append(port)
+        except (OSError, serial.SerialException):
+            pass
+    return result
+
+def autodetect_connection(serial_port=None, baudrate=None):
+    """
+    Attempt to find a connection for the receiver
+
+    :return: If found, a dictionary with the connection settings
+    """
+
+    serial_ports = [serial_port] if serial_port else __scan_serial_ports__()
+
+    if baudrate is None:
+        for serial_port in serial_ports:
+            logger.debug('Checking serial port [ {} ]'.format(serial_port))
+            baudrate = autodetect_baud_rate(serial_port)
+            if baudrate: break
+                
+    if serial_port and baudrate:
+        logger.info('u-blox found at serial [ {} ] and baudrate [ {} ]'.format(serial_port, baudrate))
+    else:
+        logger.warning('u-blox could not be found')
+
+    return {
+        SERIAL_PORT_STR: serial_port,
+        BAUD_RATE_STR: baudrate
+    }
+
+
+def __scan_serial_ports__():
     """
     Attempt to find a serial port in which the receiver is connected. 
 
     :return: If found, a string with the device specification. Otherwise, None
     """
 
-    serial_port = None
+    serial_ports = serial.tools.list_ports.comports()
 
-    for port in serial.tools.list_ports.comports():
+    serial_port = None
+    for port in serial_ports:
+        logger.debug('Checking port [ {} ]'.format(port))
         if 'u-blox GNSS receiver' in port:
+            logger.debug('Candidate port found [ {} ]'.format(port))
             serial_port = port.device
             break
 
-    return serial_port
+    if serial_port:
+        serial_ports = [serial_port]
+    else:
+        serial_ports = scan_serial_ports()
+
+    return serial_ports
 
 # ------------------------------------------------------------------------------
 
 def autodetect_baud_rate(serial_port):
    
     BAUD_RATE_LIST = [9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600]
+    PACKET_TYPE_MON_VER = core.PacketType.MON_VER
 
     baudrate = None
     
     for baud_rate_candidate in BAUD_RATE_LIST:
-        with serial.Serial(serial_port, baud_rate_candidate) as ser:
-            if ser.isOpen(): ser.close()
-            ser.open()
-            doc = ser.read(1000)
-            ser.close()
-            logger.debug(f'Testing baud rate: {baud_rate_candidate}')
+        with serial.Serial(serial_port, baud_rate_candidate, timeout=1) as serial_stream:
 
-            if b'\xb5\x62' in doc:
+            logger.debug(f'Autobauding: testing baud rate [ {baud_rate_candidate} ] and [ {serial_port} ]')
+
+            core.send(PACKET_TYPE_MON_VER, b'', serial_stream)
+            try:
+                core.wait_for_packet(PACKET_TYPE_MON_VER, serial_stream)
                 baudrate = baud_rate_candidate
-                logger.debug(f'Baud rate detected: {baudrate}')
+                logger.debug(f'Autobauding: baud rate detected [ {baudrate} ] and [ {serial_port} ]')
                 break
+            except (RuntimeError, TimeoutError):
+                logger.debug(f'Autobauding: baud rate [ {baud_rate_candidate} ]  and [ {serial_port} ] not working')
+
 
     if baudrate is None:
-        baudrate = DEFAULT_BAUD_RATE
-        logger.debug(f'Default baud rate selected: {baudrate}')
+        logger.warning(f'Autobauding: could not find a working baudrate')
 
     return baudrate
 
@@ -83,11 +149,9 @@ def submit_config(doc, serial_port=None, baudrate=None):
     format)
     """
 
-    if not serial_port:
-        serial_port = autodetect_serial_port()
-
-    if not baudrate:
-        baudrate = autodetect_baud_rate(serial_port)
+    serial_connection = autodetect_connection(serial_port, baudrate)
+    serial_port = serial_connection.get(SERIAL_PORT_STR, None)
+    baudrate = serial_connection.get(BAUD_RATE_STR, None)
 
     try:
         serial_stream = serial.Serial(serial_port, baudrate)
@@ -153,13 +217,11 @@ def detect_config(serial_port=None, baudrate=None, messages=False):
 
 def detect_connection(serial_port=None, baudrate=None):
 
-    if not serial_port:
-        logger.debug('No serial port specified, attempting to find one')
-        serial_port = autodetect_serial_port()
+    logger.debug('Detect connection')
 
-    if not baudrate:
-        logger.debug('No baud rate specified, attempting to find one')
-        baudrate = autodetect_baud_rate(serial_port)
+    serial_connection = autodetect_connection(serial_port, baudrate)
+    serial_port = serial_connection.get(SERIAL_PORT_STR, None)
+    baudrate = serial_connection.get(BAUD_RATE_STR, None)
 
     stream = serial.Serial(serial_port, baudrate=baudrate, timeout=core.TIMEOUT_DEF)
     logger.debug(f'Opened connection [ {serial_port} ] --> [ {stream} ]')
